<?php

class WC_Gateway_Westpac_Payway_Net_Directpost extends WC_Gateway_Westpac_Payway_Net {

	public function __construct() {
		
		parent::__construct();

	    $this->decrypted_results	= null;

	 	$this->caCertsFile =  dirname(__FILE__) . '/cacerts.crt';

		// Hooks
		add_action( 'woocommerce_receipt_westpac_payway_net', array(&$this, 'receipt_page') );

		add_action( 'woocommerce_api_'.strtolower(get_class($this)),array($this,'relay_response'));
		
		add_filter( 'woocommerce_thankyou_order_id',array($this,'decrypt_order_id'));
		add_filter( 'woocommerce_thankyou_order_key',array($this,'decrypt_order_key'));

	}

	/**
     * Get the order ID from the decrypted results
	 *
	 * @since 1.2.0
     */
	function decrypt_order_id($order_id) {
		global $woocommerce;
		// Decrypt the data passed back
		if (!$this->decrypted_results) {
			if (!isset($_GET['EncryptedParameters'])) return $order_id;
			$signature = (isset($_GET['Signature'])) ? $_GET['Signature'] : '';
			$this->decrypted_results = $this->decrypt_parameters($this->settings['westpac_encryption'],$_GET['EncryptedParameters'],$_GET['Signature']);
		}

		// Call the additional tasks to perform on the thank you page pre-output
		if (isset($this->decrypted_results['order'])) {
			$this->thankyou_page();
		 	return $this->decrypted_results['order'];
		}
		return $order_id;
	}

	/**
     * Get the order key from the decrypted results
	 *
	 * @since 1.2.0
     */
	function decrypt_order_key($order_key) {
		global $woocommerce;
		// Decrypt the data passed back
		if (!$this->decrypted_results) {
			if (!isset($_GET['EncryptedParameters'])) return $order_key;
			$signature = (isset($_GET['Signature'])) ? $_GET['Signature'] : '';
			$this->decrypted_results = $this->decrypt_parameters($this->settings['westpac_encryption'],$_GET['EncryptedParameters'],$_GET['Signature']);
		}

		if (isset($this->decrypted_results['key'])) return $this->decrypted_results['key'];
		return $order_key;
	}

	/**
     * Tasks to perform on the thankyou page
     * Can't do this using the thank you hooks because they occur after the template has been output
	 *
	 * @since 1.2.0
     */
	function thankyou_page() {
		if ($this->decrypted_results['summary_code'] > 0) { // Payment failed somehow.
			$order = new WC_Order((int) $this->decrypted_results['order']);
			$order->update_status( 'failed', sprintf(__("Payment couldn't be confirmed via return data: code %s - %s. \r\nTransaction ID: %s\r\n", 'woo_westpac_net'), $this->decrypted_results['response_code'], $this->decrypted_results['response_text'],$this->decrypted_results['bank_reference'] ) );
			$this->log(sprintf(__('%s PayWay payment couldn\'t be confirmed via return data for order %s. Bank response: %s - %s. Transaction ID: %s', 'woo_westpac_net'),$this->bank, $order->get_order_number(),$this->decrypted_results['response_code'], $this->decrypted_results['response_text'],$this->decrypted_results['bank_reference']));
		} elseif ($this->decrypted_results['summary_code'] == 0) {
			//$order = new WC_Order((int) $this->decrypted_results['order']);
			//if ($order->get_status() === 'failed') $order->update_status( 'pending' );
			wc_empty_cart();
		}
	}


	/**
	 * Process the payment and return the result
	 * - redirects the customer to the pay page
	 *
	 * @since 1.0.0
	 */
	function process_payment( $order_id ) {

		$order = new WC_Order( $order_id );
		$url = $order->get_checkout_payment_url( true );
		return array(
			'result' 	=> 'success',
			'redirect'	=> $url
		);
	}


	/**
	 * Collect the credit card details on the pay page and post
	 * to Westpac PayWay Net
	 * - includes token generation
	 *
	 * @since 1.0.0
	 */
	function receipt_page($order_id) {
		global $woocommerce;

		// Get the order
		$order = new WC_Order( $order_id );
		$amount = $order->get_total();

		// Payment form
		if ($this->settings['testmode']=='yes') : ?><p><?php _e('TEST MODE ENABLED', 'woothemes'); ?></p><?php endif;

		$timestamp = gmdate('YmdHis');
		$merchantid = ($this->settings['testmode']=='yes') ? 'TEST' : $this->settings['westpac_merchantid'];

		// Get the Payment Token

		if (is_woocommerce_pre_2_1()) {
			$this->return_url = get_permalink(woocommerce_get_page_id('thanks'));
			$this->return_url = add_query_arg('order','',$this->return_url);
			$this->return_url = add_query_arg('key','',$this->return_url);
			if (is_ssl()) $this->return_url = str_replace('http:','https:',$this->return_url);
		} else {
			//$this->return_url = $order->get_checkout_order_received_url();
			$this->return_url = add_query_arg('key','',add_query_arg('order','',$order->get_checkout_order_received_url()));
		}
		$this->reply_url = add_query_arg('wc-api','',$this->return_url);

		$order_key = is_callable( array( $order, 'get_order_key' ) ) ? $order->get_order_key() : $order->order_key;
		
		$token_data = array(
			'biller_code'=>$this->settings['westpac_billercode'],
			'username'=>$this->settings['westpac_username'],
			'password'=>$this->settings['westpac_password'],
			'merchant_id'=>$merchantid,
			'payment_reference'=>$order->get_order_number(),
			'payment_reference_change'=>false,
			'payment_amount'=>number_format($amount, 2, '.', ''),
			'hidden_fields'=>'order,key,wc-api',
			'order'=>$order_id,
			'key'=>$order_key,
			'wc-api'=>strtolower(get_class($this)),
			'return_link_url'=>$this->return_url,
			'return_link_redirect'=>'true',
			'reply_link_url'=>$this->reply_url,
			'reply_link_post_type'=>'extended',
			'reply_link_email'=>get_option('admin_email')
			);
		$token = $this->get_token($token_data,$order);

		if (is_callable(array($this,'get_description'))) {
            echo wpautop(wptexturize($this->get_description()));    
        } else {
            echo wpautop(wptexturize($this->description));
        } ?>
		<form method="POST" action="<?php echo $this->payurl.'MakePayment'; ?>" id="westpac_directpost_form">
		<input type="hidden" name="biller_code" value="<?php echo htmlentities($this->settings['westpac_billercode']); ?>" />
		<input type="hidden" name="token" value="<?php echo htmlentities($token); ?>" />
		<input type="hidden" name="action" value="MakePayment" />


			<fieldset>
				<p class="form-row form-row-first">
					<label for="westpac_cardholder_name"><?php _e("Name on credit card", 'woo_westpac_net') ?> <span class="required">*</span></label>
					<input type="text" class="input-text" name="nm_card_holder" id="westpac_cardholder_name" />
				</p>
				<p class="form-row form-row-first">
					<label for="westpac_card_number"><?php _e("Credit card number", 'woo_westpac_net') ?> <span class="required">*</span></label>
					<input type="text" class="input-text" name="no_credit_card" id="westpac_card_number" /><span id="jsCardType"></span>
				</p>
				<div class="clear"></div>
				<p class="form-row form-row-first">
					<label for="cc-expire-month"><?php _e("Expiration date", 'woo_westpac_net') ?> <span class="required">*</span></label>
					<select name="dt_expiry_month" id="cc-expire-month">
						<option value=""><?php _e('Month', 'woo_westpac_net') ?></option>
						<?php
							$months = array();
							for ($i = 1; $i <= 12; $i++) {
							    $timestamp = mktime(0, 0, 0, $i, 1);
							    $months[date('m', $timestamp)] = date('F', $timestamp);
							}
							foreach ($months as $num => $name) {
					            printf('<option value="%s">%s - %s</option>', $num,$num, $name);
					        }

						?>
					</select>
					<select name="dt_expiry_year" id="cc-expire-year">
						<option value=""><?php _e('Year', 'woo_westpac_net') ?></option>
						<?php
							$years = array();
							for ($i = date('Y'); $i <= date('Y') + 15; $i++) {
							    printf('<option value="%u">%u</option>', $i, $i);
							}
						?>
					</select>
				</p>
				<p class="form-row form-row-last">
					<label for="westpac_card_ccv"><?php _e("Card security code", 'woo_westpac_net') ?> <span class="required">*</span></label>
					<input type="text" class="input-text" id="westpac_card_ccv" name="no_cvn" maxlength="4" style="width:45px" />
					<span class="help westpac_card_ccv_description"><?php _e('3 or 4 digits usually found on the signature strip.', 'woo_westpac_net') ?></span>
				</p>
				<div class="clear"></div>
			</fieldset>
			<input type="submit" id="jsPayButton" class="submit buy button" value="<?php _e('Confirm and pay','woo_westpac_net'); ?>" />
			</form>
			<script type="text/javascript">
			jQuery(function(){
                jQuery('form#westpac_directpost_form').on('submit',function() {
                    jQuery(this).block({
                        message: null,
                        overlayCSS: {
                            background: '#fff',
                            opacity: 0.6
                        }
                    });
                    return true;
                });
				jQuery('input#jsPayButton').attr('disabled', 'disabled');
				jQuery('input#westpac_card_number').keyup(function() {
					var number = jQuery(this).val();
					number = number.replace(/[^0-9]/g, '');
						var re = new RegExp("^4[0-9]{12}(?:[0-9]{3})?$");
       					if (number.match(re) != null) {
          				jQuery('span#jsCardType').html('<img src="<?php echo WP_PLUGIN_URL . "/" . plugin_basename( dirname(__FILE__)) . '/images/visa.png'; ?>" alt="Visa detected" style="vertical-align: bottom;"/>');
			            jQuery('input#jsCardType').val('visa');
			            jQuery('input#jsPayButton').removeAttr('disabled');
			            return;
       					}
						re = new RegExp("^(?:5[1-5][0-9]{2}|222[1-9]|22[3-9][0-9]|2[3-6][0-9]{2}|27[01][0-9]|2720)[0-9]{12}$");
			            if (number.match(re) != null) {
			            jQuery('span#jsCardType').html('<img src="<?php echo WP_PLUGIN_URL . "/" . plugin_basename( dirname(__FILE__)) . '/images/mastercard.png'; ?>" alt="Mastercard detected" style="vertical-align: bottom;"/>');
			            jQuery('input#jsCardType').val('mastercard');
			            jQuery('input#jsPayButton').removeAttr('disabled');
			            return;
			            }
			            re = new RegExp("^3[47][0-9]{13}$");
			            if (number.match(re) != null) {
			            jQuery('span#jsCardType').html('<img src="<?php echo WP_PLUGIN_URL . "/" . plugin_basename( dirname(__FILE__)) . '/images/amex.png'; ?>" alt="American Express detected"  style="vertical-align: bottom;"/>');
			            jQuery('input#jsCardType').val('amex');
			            jQuery('input#jsPayButton').removeAttr('disabled');
			            return;
			            }
			            re = new RegExp("^3(?:0[0-5]|[68][0-9])[0-9]{11}$");
			            if (number.match(re) != null) {
			            jQuery('span#jsCardType').html('<img src="<?php echo WP_PLUGIN_URL . "/" . plugin_basename( dirname(__FILE__)) . '/images/diners.png'; ?>" alt="Diners Club card detected"  style="vertical-align: bottom;"/>');
			            jQuery('input#jsCardType').val('dinersclub');
			            jQuery('input#jsPayButton').removeAttr('disabled');
			            return;
			            }
			            re = new RegExp("^(?:3[0-9]{15}|(2131|1800)[0-9]{11})$");
			            if (number.match(re) != null) {
			            jQuery('span#jsCardType').html('<img src="<?php echo WP_PLUGIN_URL . "/" . plugin_basename( dirname(__FILE__)) . '/images/jcb.png'; ?>" alt="JCB card detected"  style="vertical-align: bottom;"/>');
			            jQuery('input#jsCardType').val('jcb');
			            jQuery('input#jsPayButton').removeAttr('disabled');
			            return;
			            }
						jQuery('span#jsCardType').html('');
						jQuery('input#jsCardType').val('');
				});
			});
			</script>
	<?php
	}

	/*
     * Request a secure token from the PayWay server to protect these payment details.
     * $parameters is the array that maps payment parameter names to parameter values.
     * The return value is the token value that should be included in the hand-off.
     * This function modified from Westpac's sample code (token.php)
     */
    function get_token( $parameters, $order ) {

        // Find the port setting, if any.
        $payWayUrl = $this->payurl;
        $port = 443;
        $portPos = strpos( $this->payurl, ":", 6 );
        $urlEndPos = strpos( $this->payurl, "/", 8 );
        if ( $portPos !== false && $portPos < $urlEndPos )
        {
            $port = (int)substr( $this->payurl, ((int)$portPos) + 1, ((int)$urlEndPos));
            $payWayUrl = substr( $this->payurl, 0, ((int)$portPos))
                . substr( $this->payurl, ((int)$urlEndPos), strlen($this->payurl));
        }

        $ch = curl_init( $payWayUrl . "RequestToken" );

        if ( $port != 443 )
        {
            curl_setopt( $ch, CURLOPT_PORT, $port );
        }

        curl_setopt( $ch, CURLOPT_FAILONERROR, true );
        curl_setopt( $ch, CURLOPT_FORBID_REUSE, true );
        curl_setopt( $ch, CURLOPT_FRESH_CONNECT, true );
        curl_setopt( $ch, CURLOPT_RETURNTRANSFER, true );

        // Set timeout options
        curl_setopt( $ch, CURLOPT_CONNECTTIMEOUT, 30 );
        curl_setopt( $ch, CURLOPT_TIMEOUT, 30 );

        // Set references to certificate files
        curl_setopt( $ch, CURLOPT_CAINFO, $this->caCertsFile );

        // Check the existence of a common name in the SSL peer's certificate
        // and also verify that it matches the hostname provided
        curl_setopt( $ch, CURLOPT_SSL_VERIFYHOST, 2 );

        // Verify the certificate of the SSL peer
        curl_setopt( $ch, CURLOPT_SSL_VERIFYPEER, true );

        // Build the parameters string to pass to PayWay
        $parametersString = '';
        $init = true;
        foreach ( $parameters as $paramName => $paramValue )
        {
            if ( $init )
            {
            		$init = false;
            }
            else
            {
                $parametersString = $parametersString . '&';
            }
            $parametersString = $parametersString . urlencode($paramName) . '=' . urlencode($paramValue);
        }

        curl_setopt( $ch, CURLOPT_POSTFIELDS, $parametersString );

        // Make the request
        $responseText = curl_exec($ch);

        // Check the response for errors
        $errorNumber = curl_errno( $ch );

        if ( $errorNumber != 0 )
        {
            trigger_error( "CURL Error getting token: Error Number: " . $errorNumber .
                ", Description: '" . curl_error( $ch ) . "'" );
            $this->log(sprintf(__('CURL Error getting token for order ID %s. Error: %s - %s', 'woo_westpac_net'),$order->get_order_number(),$errorNumber,curl_error( $ch )));
        }

        curl_close( $ch );

        // Split the response into parameters
        $responseParameterArray = explode( "&", $responseText );
        $responseParameters = array();
        foreach ( $responseParameterArray as $responseParameter )
        {
            list( $paramName, $paramValue ) = explode( "=", $responseParameter, 2 );
            $responseParameters[ $paramName ] = $paramValue;
        }

        if ( array_key_exists( 'error', $responseParameters ) )
        {
        	echo '<p class="woocommerce-error">'.__('Error getting token: ','woo_westpac_net').$responseParameters['error'].'</p>';
        	$this->log(sprintf(__('Error getting token for order ID %s. Error: %s', 'woo_westpac_net'),$order->get_order_number(),$responseParameters['error']));
        }
        else
        {
            return $responseParameters['token'];
        }
    }

	/**
	 * Helper function for decryption
	 */
	function pkcs5_unpad($text)
    {
        $pad = ord($text{strlen($text)-1});
        if ($pad > strlen($text)) return false;
        if (strspn($text, chr($pad), strlen($text) - $pad) != $pad) return false;
        return substr($text, 0, -1 * $pad);
    }

	/**
	 * Decrypt the parameters passed back on the return_url (to the woocommerce thankyou page)
	 */
    function decrypt_parameters( $base64Key, $encryptedParametersText, $signatureText )
    {
        $key = base64_decode( $base64Key );
        $iv = "\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0\0";
        if (function_exists('mcrypt_module_open')) {
            $td = mcrypt_module_open('rijndael-128', '', 'cbc', '');

            // Decrypt the parameter text
            mcrypt_generic_init($td, $key, $iv);
            $parametersText = mdecrypt_generic($td, base64_decode( $encryptedParametersText ) );
            $parametersText = $this->pkcs5_unpad( $parametersText );
            mcrypt_generic_deinit($td);
            // Decrypt the signature value
            mcrypt_generic_init($td, $key, $iv);
            $hash = mdecrypt_generic($td, base64_decode( $signatureText ) );
            $hash = bin2hex( $this->pkcs5_unpad( $hash ) );
            mcrypt_generic_deinit($td);
            mcrypt_module_close($td);
        } else {
            $parametersText = openssl_decrypt(base64_decode($encryptedParametersText), 'AES-128-CBC', $key, OPENSSL_RAW_DATA, $iv);
            $hash = openssl_decrypt(base64_decode($signatureText), 'AES-128-CBC', $key, OPENSSL_RAW_DATA, $iv);
            $hash = bin2hex($hash);
        }
            
        // Compute the MD5 hash of the parameters
        $computedHash = md5( $parametersText );

        // Check the provided MD5 hash against the computed one
        if ( $computedHash != $hash )
        {
            trigger_error( "Invalid parameters signature" );
        }
        $parameterArray = explode( "&", $parametersText );
        $parameters = array();

        // Loop through each parameter provided
        foreach ( $parameterArray as $parameter )
        {
            list( $paramName, $paramValue ) = explode( "=", $parameter );
            $parameters[ urldecode( $paramName ) ] = urldecode( $paramValue );
        }
        return $parameters;
    }

	/**
	 * Relay response - handles response from Westpac PayWay Net
	 * PayWay strips query args for server to server payment notification, so we'll check against the _POST data
	 * @since 1.0.0
	 */
	function relay_response() {
		global $woocommerce;
		if (!isset($_POST['cd_summary']) || (!isset($_POST['wc-api']) || $_POST['wc-api'] !== strtolower(get_class($this)))) return;

		// Process response
		$response = new stdClass;
		foreach ($_GET as $key => $value) {
			$response->$key = $value;
		}
	    foreach ($_POST as $key => $value) {
            $response->$key = $value;
        }

        $order = new WC_Order((int) $response->order);

		if ($response->cd_summary == 0) { // Approved
			if ($order->key_is_valid( $response->key )) {

				// Payment complete
				$order->add_order_note(
				'Payment received: '.woocommerce_price($response->am_payment)
				."\r\n".$this->bank." CC Number: ".$response->TruncatedCardNumber
				."\r\n".$this->bank." CC Name: ".$response->nm_card_holder
				."\r\n".$this->bank." CC Type: ".$response->nm_card_scheme
				."\r\n".$this->bank." Receipt Number: ".$response->no_receipt
				."\r\n".$this->bank." Payment Time: ".$response->ti_payment
				."\r\n".$this->bank." Settlement date: ".$response->dt_payment);

				$order->payment_complete();

				// Remove cart
				wc_empty_cart();
				die();

			} else { // payment received but order key didn't match!

				// Key did not match order id
				$order->add_order_note( sprintf(__('Payment received ('.woocommerce_price($response->am_payment).'), but order key returned was not valid: code %s - %s. You should check this payment matches correctly before processing.', 'woo_westpac_net'), $response->cd_response, $response->tx_response ) );

				$this->log(sprintf(__('Payment received (%s) for order %s, but order key returned was not valid: code %s - %s. You should check this payment matches correctly before processing.', 'woo_westpac_net'),woocommerce_price($response->am_payment),$order->get_order_number(),$response->cd_response, $response->tx_response));

				// Put on hold if pending
				if ($order->status == 'pending' || $order->status == 'failed') {
					$order->update_status( 'on-hold' );
				}
				die();
			}
		} else { // Transaction failed
			$order->add_order_note(sprintf(__($this->bank." payment attempted, but FAILED: code %s - %s. \r\Receipt number: %s\r\n", 'woo_westpac_net'), $response->cd_response, $response->tx_response,$response->no_receipt ) );
			$order->update_status( 'failed' );
			$this->log(sprintf(__('%s payment attempted for order %s, but FAILED: code %s - %s. Receipt number: %s', 'woo_westpac_net'),$this->bank,$order->get_order_number(),$response->cd_response, $response->tx_response, $response->no_receipt ));
			die();
		}

		http_response_code(500);
		die();
	}

}