<?php

class WC_Gateway_Westpac_Payway_Net_Trustedframe extends WC_Gateway_Westpac_Payway_Net {

    public function __construct() {
        
        parent::__construct();

        add_action('wp_enqueue_scripts', array($this,'load_checkout_script'));

        $this->has_fields = true;

        $this->apibase = 'https://api.payway.com.au/rest/v1/';

        $this->renew_api_key();

    }

    /**
     * Load checkout script
     */
    function load_checkout_script() {
        if (is_checkout() || is_wc_endpoint_url( 'order-pay' )) {
        //if (is_checkout()) {
            $script_url = 'https://api.payway.com.au/rest/v1/payway.js';
            $publishable_key = $this->settings['publishable_key'];
            $styles = ($this->isJSON('{ '.$this->settings['trustedframe_styles'].' }')) ? $this->settings['trustedframe_styles'] : '';

            wp_enqueue_script('woo_westpac-net_script',$script_url,array('jquery','woocommerce','wc-checkout'),false,true);

            wp_add_inline_script('woo_westpac-net_script',"
                
                paywayFrame = null;

                var tokenCallback = function(err, data) {
                    if (!err) {
                        jQuery('#westpac-net-token').val(data.singleUseTokenId);
                        var form = jQuery('form.checkout');
                        if (jQuery('form#order_review').length) {
                            form = jQuery('form#order_review');
                        }
                        form.removeClass('do-westpac-token');
                        form.submit();
                        setTimeout(function() {
                            form.addClass('do-westpac-token');
                        },1000);
                    } else {
                        jQuery('#payway-credit-card').after('<div id=\'payway-error-msg\' class=\'woocommerce-error\'>".__('Card could not be processed. Please confirm your credit card details.','woo_westpac_net')."</div>');
                        return false;
                    }
                }

                var createdCallback = function(err, frame) {
                    paywayFrame = frame;
                    jQuery('.westpac-loading-frame').hide();
                    if (!err) {
                        var form = jQuery('form.checkout');
                        if (jQuery('form#order_review').length) {
                            form = jQuery('form#order_review');
                        }
                        form.addClass('do-westpac-token');
                        form.on('checkout_place_order_westpac_payway_net',function() { 
                            jQuery('#payway-error-msg').remove();
                            if (form.hasClass('do-westpac-token')) {
                                frame.getToken(tokenCallback);
                                return false;
                            } else {
                                return true;
                            }
                        });
                        if (jQuery('form#order_review').length) {
                            form.on('click','#place_order',function(e) { 
                                jQuery('#payway-error-msg').remove();
                                if (form.hasClass('do-westpac-token')) {
                                    frame.getToken(tokenCallback);
                                    e.preventDefault();
                                    return false;
                                } else {
                                    return true;
                                }
                            });
                        } 
                    } else {
                        // Couldn't create frame
                    }
                }

                jQuery('body').on('updated_checkout', function() {
                    initWestpacTrustedFrame();
                });

                if (jQuery('body').hasClass('woocommerce-order-pay')) {
                    initWestpacTrustedFrame();
                }

                function initWestpacTrustedFrame() {
                    if (typeof payway == 'undefined') {
                        setTimeout(function() {
                            initWestpacTrustedFrame();
                            }, 1000);
                    }
                    if (paywayFrame !== null) paywayFrame.destroy();
                    var submit = document.getElementById('place_order');
                    payway.createCreditCardFrame({
                        publishableApiKey: '$publishable_key',
                        onValid: function() { submit.disabled = false; },
                        onInvalid: function() { submit.disabled = true; },
                        tokenMode: 'callback',
                        layout: 'narrow',
                        style: { $styles }
                    },createdCallback);
                }
                ");

        }
    }

    function payment_fields() {
        if (is_callable(array($this,'get_description'))) {
            echo wpautop(wptexturize($this->get_description()));    
        } else {
            echo wpautop(wptexturize($this->description));
        }
        echo '<input type="hidden" name="westpac-token" id="westpac-net-token" />';
        echo '<div class="westpac-loading-frame">'.__('Loading...','woo_westpac_net').'</div>';
        echo '<div id="payway-credit-card"></div>';
    }

    function isJSON($string) {
        return is_string($string) && is_array(json_decode($string, true)) && (json_last_error() == JSON_ERROR_NONE) ? true : false;
    }

    

    /**
     * Process the payment and return the result
     * - redirects the customer to the pay page
     *
     * @since 1.0.0
     */
    function process_payment( $order_id ) {
        
        global $woocommerce;
        $order = new WC_Order( $order_id );

        $card_token = isset( $_POST['westpac-token'] ) ? wc_clean( $_POST['westpac-token'] ) : '';
        if (version_compare( WC_VERSION, '2.7', '<' )) {
            $billing_email = $order->billing_email;
            $amount = $order->order_total;
        } else {
            $billing_email = $order->get_billing_email();
            $amount = $order->get_total();
        }

        $customer_number = time();

        $paydata = array(
            'singleUseTokenId'  => $card_token,
            'customerNumber'    => $customer_number,
            'transactionType'   => 'payment',
            'principalAmount'   => $amount,
            'currency'          => strtolower(get_woocommerce_currency()),
            'orderNumber'       => $order->get_order_number(),
            'merchantId'        => $this->settings['westpac_merchantid']
        );

        $result = $this->call_api($paydata,'transactions', $order);

        if (isset($result->data) && is_array($result->data)) {
            foreach ($result->data as $err) {
                $order->add_order_note(sprintf(__('PayWay error in field %s: %s', 'woo_westpac_net'), $err->fieldName, $err->message));
                wc_add_notice(sprintf(__('PayWay error in field %s: %s', 'woo_westpac_net'), $err->fieldName, $err->message),'error');
            }
            return array(
                'result' => 'failure',
                'redirect' => $order->get_checkout_payment_url(true)
            );
        }

        if (isset($result->status) && $result->status === "approved") {
            // Success
            $order->add_order_note(sprintf(__('PayWay payment on card %s approved at %s. Reference ID: %s','woo_westpac_net'),$result->creditCard->cardNumber,$result->transactionDateTime,$result->transactionId));
            $order->payment_complete($result->transactionId);
            $woocommerce->cart->empty_cart();

            if (is_woocommerce_pre_2_1()) {
                $redirect = add_query_arg('key', $order->order_key, add_query_arg('order', $order_id, get_permalink(get_option('woocommerce_thanks_page_id'))));
            } else { // WC 2.1+
                $redirect = $this->get_return_url($order);
            }
            return array(
                'result' => 'success',
                'redirect' => $redirect
            );
        } elseif (isset($result->status) && $result->status !== "approved") {
            // Failed
            $order->add_order_note(sprintf(__('PayWay payment failed: %s. (ref ID: %s)', 'woo_westpac_net'), $result->responseText, $result->transactionId));
            wc_add_notice(sprintf(__('PayWay payment failed: %s', 'woo_westpac_net'), $result->responseText),'error');
            $this->log(sprintf(__('Order %s: PayWay payment failed - %s. Reference ID: %s', 'woo_westpac_net'),$order->get_order_number(),$result->responseText, $result->transactionId));
        } else {
            // Errored
            $order->add_order_note(sprintf(__('PayWay payments error: %s', 'woo_westpac_net'), $result->message));
            wc_add_notice(sprintf(__('Payment error: %s', 'woo_westpac_net'), $result->message),'error');
            $this->log(sprintf(__('Order %s: PayWay payment error - %s.', 'woo_westpac_net'),$order->get_order_number(),$result->message));
        }
        return array(
            'result' => 'failure',
            'redirect' => $order->get_checkout_payment_url(true)
        );

    }

    function call_api($post_data,$action='transactions', $order=null, $method = 'POST') {
        global $woocommerce;

        $secret_key = $this->settings['secret_key'];

        $response = wp_remote_post( $this->apibase.$action , array(
            'method'        => $method,
            'headers' => array(
                'Authorization' => 'Basic ' . base64_encode( $secret_key . ':' )
            ),
            'body'          => $post_data,
            'timeout'       => 70,
            'user-agent'    => 'WooCommerce ' . $woocommerce->version
        ));
        if (is_wp_error($response)) {
            $result = $response;
            $order_msg = '';
            if ($order) {
                $order_msg = 'Order '.$order->get_order_number();
            }
            $this->log(sprintf(__('Payway Rest API error - %s %s.', 'woo_westpac_net'),$order_msg,$response->get_error_message()));
        } else {
            $result = json_decode($response['body']);
        }
        return $result;
    }

    function renew_api_key() {
        // When was it last updated
        $updated = get_option('westpac_api_key_last_updated');
        if ($updated == false || $updated < (strtotime('24 hours ago'))) {
            // Update key
            $result = $this->call_api(array(),'api-keys/latest',null,'GET');
            if (isset($result->key)) {
                $settings = get_option('woocommerce_westpac_payway_net_settings');
                $settings['secret_key'] = $result->key;
                update_option('woocommerce_westpac_payway_net_settings',$settings);
                update_option('westpac_api_key_last_updated',time());    
            }
        }
    }

}