<?php
/*
Plugin Name: WooCommerce Westpac & St. George PayWay NET Gateway
Plugin URI: http://woothemes.com/woocommerce
Woo: 185011:2193bfa6dee6fedb04d81f98a0f21e75
Description: Use Westpac or St. George PayWay Net (trusted frame & direct post) as a credit card processor for WooCommerce.
Version: 2.0.10
Author: Tyson Armstrong
Author URI: http://tysonarmstrong.com/

Copyright: © 2013-2017 Tyson Armstrong

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License, version 2, as
published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

/**
 * Required functions
 */
if ( ! function_exists( 'woothemes_queue_update' ) )
	require_once( 'woo-includes/woo-functions.php' );

/**
 * Plugin updates
 */
woothemes_queue_update( plugin_basename( __FILE__ ), '2193bfa6dee6fedb04d81f98a0f21e75', '185011' );

add_action('plugins_loaded', 'woocommerce_westpac_payway_net_init', 0);

function woocommerce_westpac_payway_net_init() {

	if (!class_exists('WC_Payment_Gateway'))  return;

	class WC_Gateway_Westpac_Payway_Net extends WC_Payment_Gateway {

		private static $log;

		public function __construct() {
			global $woocommerce;

		    $this->id 					= 'westpac_payway_net';
		    $this->method_title 		= __('Westpac & St. George PayWay Net', 'woothemes');
			$this->method_description 	= __('Westpac & St. George PayWay Net handles all the steps in the secure transaction while remaining virtually transparent. Payment data is passed from the checkout form to Westpac or St. George PayWay Net for processing thus removing the complexity of PCI compliance.', 'woothemes');

			$this->do_version_update();

		    // Load the form fields.
		    $this->init_form_fields();

		    // Load the settings.
		    $this->init_settings();

		 	$this->payurl = 'https://www.payway.com.au/';
		 	if ($this->settings['bank'] == "stgeorge") $this->payurl = 'https://payway.stgeorge.com.au/';

		 	// Bank name
		 	$this->bank = "Westpac";
		 	if ($this->settings['bank'] == "stgeorge") $this->bank = "St. George";

		 	// Setup icon
		 	$this->icon = WP_PLUGIN_URL . "/" . plugin_basename( dirname(__FILE__)) . '/images/westpac_small.png';
		 	if ($this->settings['bank'] == "stgeorge") $this->icon = WP_PLUGIN_URL . "/" . plugin_basename( dirname(__FILE__)) . '/images/stgeorge_small.png';

		    // Define user set variables
		    $this->title = $this->settings['title'];
		    $this->description = 'Credit cards accepted: Visa, Mastercard';
		    if ($this->settings['accept_amex'] == 'yes') $this->description .= ', American Express';
		    if ($this->settings['accept_diners'] == 'yes') $this->description .= ', Diners Club';
		    if ($this->settings['accept_jcb'] == 'yes') $this->description .= ', JCB';


			// Save admin options
			/* 1.6.6 */
			add_action( 'woocommerce_update_options_payment_gateways', array( &$this, 'process_admin_options' ) );
			/* 2.0.0 */
			add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );

		}

		/**
	     * Initialise Gateway Settings Form Fields
		 *
		 * @since 1.0.0
	     */
		function init_form_fields() {
			$this->form_fields = array(
			    'enabled' => array(
			        'title' => __( 'Enable/Disable', 'woothemes' ),
			        'type' => 'checkbox',
			        'label' => __( 'Enable this payment method', 'woothemes' ),
			        'default' => 'yes'
			    ),
			    'api' => array(
					'title' => __( 'Which API method', 'woothemes' ),
					'type' => 'select',
					'options' => array('trustedframe' => 'Trusted Frame', 'directpost' => 'Direct Post' ),
					'description' => __( 'Use Trusted Frame for new installations. Use Direct Post if you\'ve been using this gateway prior to version 2.', 'woothemes' ),
					'default' => 'trustedframe'
				),
			    'title' => array(
			        'title' => __( 'Title', 'woothemes' ),
			        'type' => 'text',
			        'description' => __( 'This is the title the customer will see during checkout.', 'woothemes' ),
			        'default' => __( 'Westpac & St. George PayWay Net', 'woothemes' )
			    ),
				'testmode' => array(
					'title' => __( 'Test mode', 'woothemes' ),
					'label' => __( 'Enable Test mode', 'woothemes' ),
					'type' => 'checkbox',
					'description' => __( 'Process transactions in Test mode. No transactions will actually take place.', 'woothemes' ),
					'default' => 'yes',
					'class' => 'for-directpost'
				),
				'bank' => array(
					'title' => __( 'Bank', 'woothemes' ),
					'type' => 'select',
					'options' => array('westpac' => 'Westpac', 'stgeorge' => 'St. George' ),
					'description' => __( 'Select which bank is supplying your merchant account.', 'woothemes' ),
					'default' => 'westpac',
				),
				'westpac_billercode' => array(
					'title' => __( 'PayWay Biller Code', 'woothemes' ),
					'type' => 'text',
					'description' => __( 'A six-digit number available from <a href="https://www.payway.com.au" target="_blank">Westpac</a> / <a href="https://payway.stgeorge.com.au" target="_blank">St. George</a>.', 'woothemes' ),
					'default' => '',
					'class' => 'for-directpost'
				),
				'westpac_merchantid' => array(
					'title' => __( 'Merchant ID', 'woothemes' ),
					'type' => 'text',
					'description' => __( 'Available from <a href="https://www.payway.com.au" target="_blank">Westpac</a> / <a href="https://payway.stgeorge.com.au" target="_blank">St. George</a>.', 'woothemes' ),
					'default' => 'TEST'
				),
				'westpac_username' => array(
					'title' => __( 'PayWay Username', 'woothemes' ),
					'type' => 'text',
					'description' => __( 'A six-characer, alphanumeric string available from <a href="https://www.payway.com.au" target="_blank">Westpac</a> / <a href="https://payway.stgeorge.com.au" target="_blank">St. George</a>.', 'woothemes' ),
					'default' => '',
					'class' => 'for-directpost'
				),
				'westpac_password' => array(
					'title' => __( 'PayWay Password', 'woothemes' ),
					'type' => 'password',
					'description' => __( 'Available from <a href="https://www.payway.com.au" target="_blank">Westpac</a> / <a href="https://payway.stgeorge.com.au" target="_blank">St. George</a>.', 'woothemes' ),
					'default' => '',
					'class' => 'for-directpost'
				),
				'westpac_encryption' => array(
					'title' => __( 'Encryption Key', 'woothemes' ),
					'type' => 'password',
					'description' => __( 'Available from <a href="https://www.payway.com.au" target="_blank">Westpac</a> / <a href="https://payway.stgeorge.com.au" target="_blank">St. George</a>.', 'woothemes' ),
					'default' => '',
					'class' => 'for-directpost'
				),
				'publishable_key' => array(
					'title' => __( 'Publishable API Key', 'woothemes' ),
					'type' => 'password',
					'description' => __( 'Available from <a href="https://www.payway.com.au" target="_blank">Westpac</a> / <a href="https://payway.stgeorge.com.au" target="_blank">St. George</a>.', 'woothemes' ),
					'default' => '',
					'class' => 'for-trustedframe'
				),
				'secret_key' => array(
					'title' => __( 'Secret API Key', 'woothemes' ),
					'type' => 'password',
					'description' => __( 'Available from <a href="https://www.payway.com.au" target="_blank">Westpac</a> / <a href="https://payway.stgeorge.com.au" target="_blank">St. George</a>.', 'woothemes' ),
					'default' => '',
					'class' => 'for-trustedframe'
				),
				'trustedframe_styles' => array(
					'title' => __( 'Override Trusted Frame Styles', 'woocommerce' ),
			        'type' => 'textarea',
			        'description' => __( 'Refer to the <a href="https://www.payway.com.au/docs/rest.html#style" target="_blank">documentation</a> to style the credit card form.', 'woocommerce' ),
			        'default' => __('".payway-card" : {
    "border": "0",
    "background": "transparent"
}', 'woocommerce'),
					'class' => 'for-trustedframe'
				),
				'accept_amex' => array(
					'title' => __( 'Accept American Express', 'woothemes' ),
					'label' => __( 'Accept American Express cards', 'woothemes' ),
					'type' => 'checkbox',
					'description' => __( 'Call 1300 363 614 to activate American Express on your account.', 'woothemes' ),
					'default' => 'no'
				),
				'accept_diners' => array(
					'title' => __( 'Accept Diners Club', 'woothemes' ),
					'label' => __( 'Accept Diners Club cards', 'woothemes' ),
					'type' => 'checkbox',
					'description' => __( 'Call 1300 360 500 to activate Diners Club on your account.', 'woothemes' ),
					'default' => 'no'
				),
				'accept_jcb' => array(
					'title' => __( 'Accept JCB', 'woothemes' ),
					'label' => __( 'Accept JCB cards', 'woothemes' ),
					'type' => 'checkbox',
					'description' => __( 'Call 1300 363 614 to activate JCB on your account.', 'woothemes' ),
					'default' => 'no'
				)
			);
		} // End init_form_fields()

		/**
		 * Admin Panel Options
		 *
		 * @since 1.0.0
		 */
		public function admin_options() {

	    	?>
	    	<h3><?php _e('Westpac & St George PayWay Net Credit Card Payment', 'woo_westpac_net'); ?></h3>
	    	<p><?php _e('Using the Westpac or St. George PayWay Net payment gateway.', 'woo_westpac_net'); ?></p>
			<p><?php _e('If using Direct Post, you\'ll need to add your server IP to your <a href="https://www.payway.com.au" target="_blank">Westpac</a> / <a href="https://payway.stgeorge.com.au" target="_blank">St. George</a> Payway account. Go to Setup Net > Hosted Payment Page and add the IP under <strong>IP Address</strong>. ', 'woo_westpac_net');
	    	$external_ip = $this->get_external_ip();
	    	if ($external_ip) {
	    		_e('Your server\'s IP address is:', 'woo_westpac_net'); 
	    		echo ' <strong>'.$external_ip.'</strong>';
	    		_e(' (checks after 15 minutes)', 'woo_westpac_net');
	    	} ?></p>
	    	<table class="form-table westpac-net-settings">
	    	<?php
	    		// Generate the HTML For the settings form.
	    		$this->generate_settings_html();
	    	?>
			</table><!--/.form-table-->

			<script type="text/javascript">
				function westpac_show_relevant_fields() {
					var api = jQuery('select#woocommerce_westpac_payway_net_api').val();
					jQuery('table.westpac-net-settings tr').show();
					if (api === 'trustedframe') {
						jQuery('.for-directpost').closest('tr').hide();
					} else {
						jQuery('.for-trustedframe').closest('tr').hide();
					}
				}
				westpac_show_relevant_fields();
				jQuery('select#woocommerce_westpac_payway_net_api').on('change', function() {
					westpac_show_relevant_fields();
				});
			</script>
	    	<?php
	    } // End admin_options()

	    /**
	     * Get external IP to show in settings
		 *
		 * @since 1.2.0
	     */
		function get_external_ip() {
			$external_ip = get_option('_external_ip');
			$external_ip_last_updated = get_option('_external_ip_last_updated');
			if ($external_ip == false || $external_ip_last_updated < strtotime('- 15 minutes')) {
				// Check IP
				$response = wp_remote_get('https://ifconfig.co/ip');
				if (is_array($response)) {
					$responseText = $response['body'];
					if (filter_var($responseText, FILTER_VALIDATE_IP)) {
						update_option('_external_ip',trim($responseText));
						update_option('_external_ip_last_updated',time());
					}
					return trim($responseText);
				} else {
					return false;
				}
			} else {
				// Here's the stored IP
				return $external_ip;
			}
			return false;
		}

		public static function log( $message ) {
			if ( empty( self::$log ) ) {
				self::$log = new WC_Logger();
			}

			self::$log->add( 'woo_westpac_net', $message );

		}

		private function do_version_update() {
			$settings = get_option('woocommerce_westpac_payway_net_settings');
			if ((!isset($settings['api']) || $settings['api'] === '') && isset($settings['westpac_billercode']) && $settings['westpac_billercode'] !== '') {
				$settings['api'] = 'directpost';
				update_option('woocommerce_westpac_payway_net_settings',$settings);
			}
		}

	}

	if (!function_exists('is_woocommerce_pre_2_1')) {
		function is_woocommerce_pre_2_1() {
	        if ( ! defined( 'WC_VERSION' ) ) {
	            $woocommerce_is_pre_2_1 = true;
	        } else {
	            $woocommerce_is_pre_2_1 = false;
	        }
	        return $woocommerce_is_pre_2_1;
	    }
	}


	/**
	 * Add the Westpac PayWay Net gateway to WooCommerce
	 *
	 * @since 1.0.0
	 **/
	function add_westpac_payway_net_gateway( $methods ) {
		// Get options to determine which version to install
		$settings = get_option('woocommerce_westpac_payway_net_settings');
		if ($settings['api'] === 'directpost') {
			include_once( 'woocommerce-westpac-net-directpost.php' );
			$methods[] = 'WC_Gateway_Westpac_Payway_Net_Directpost';	
		} else {
			include_once( 'woocommerce-westpac-net-trustedframe.php' );
			$methods[] = 'WC_Gateway_Westpac_Payway_Net_Trustedframe';	
		}
		return $methods;
	}
	add_filter('woocommerce_payment_gateways', 'add_westpac_payway_net_gateway' );
}